this.name        = "Repair Nanobots";
this.author      = "Thargoid";
this.copyright   = "Creative Commons Attribution - Non-Commercial - Share Alike 3.0 license with clauses - see readme.txt.";
this.description = "Scan through a list of possible broken equipment, see what the player has, and then fix one of them";
this.version     = "2.11";
"use strict";

//------------------------------------------------------------------------------------------//
this.$repairSystems = function() {
    var _fname = "repairSystems";
    this.$playerDamagedList = [];
    var _ship = player.ship;
    var _equipment = _ship.equipment;
    var _listCounter = 0 ; // reset the counter
    var _eqInfo;

    for(_listCounter = 0; _listCounter < _equipment.length; _listCounter++) {
        if (_ship.equipmentStatus(equipment[_listCounter].equipmentKey) !== "EQUIPMENT_DAMAGED")
            continue; 
        _eqInfo = _equipment[_listCounter];
        log(this.name, _fname+": Equipment damaged:"+_eqInfo.equipmentKey+", RepairBotChance:"+_eqInfo.scriptInfo.thargoidRepairBotChance);
        if (_eqInfo.scriptInfo.thargoidRepairBotChance === undefined 
                || isNaN(_eqInfo.scriptInfo.thargoidRepairBotChance)
                || (!isNaN(_eqInfo.scriptInfo.thargoidRepairBotChance) && _eqInfo.scriptInfo.thargoidRepairBotChance > 0) ) { 
            // if it's broke and fixable, add it to the list.
            log(this.name, _fname+": Putting damaged equipment in repair list:"+_eqInfo.equipmentKey);
            this.$playerDamagedList.push(eqInfo.equipmentKey); 
        }
    }
 
    if (this.$playerDamagedList.length === 0) {
        log(this.name, _fname+": Nothing damaged that the repair bots can fix.");
        player.consoleMessage("Nothing damaged that the repair bots can fix.", 5);
        return;
    } else {
        var damagedEquipment = Math.floor(Math.random() * this.$playerDamagedList.length); // pick a random element from the list...
        _fixedItem = this.$playerDamagedList[damagedEquipment]; // ...define the item...
        _eqInfo = EquipmentInfo.infoForKey(_fixedItem);
        _fixedName = _eqInfo.name; // define it's screen name
        let _fixedTech = _eqInfo.effectiveTechLevel // tech level of the item
        log(this.name, _fname+": Choose "+_fixedItem+" to repair from list: "+this.$playerDamagedList);
        log(this.name, _fname+": Choosen equipment "+_fixedItem+" RepairBotChance: "+_eqInfo.scriptInfo.thargoidRepairBotChance+", fixedTech: "+_fixedTech+", Name: "+_fixedName);

        switch (true) {
            case (_eqInfo.scriptInfo.thargoidRepairBotChance !== undefined && !isNaN(_eqInfo.scriptInfo.thargoidRepairBotChance)): {
                let _fixChance = _eqInfo.scriptInfo.thargoidRepairBotChance;
                break;
            }
            case (_fixedTech < 9): { 
                let _fixChance = 1; 
                break; 
            }
            case (_fixedTech > 8 && _fixedTech < 17): {
                let _fixChance = 1 - ((_fixedTech - 8)/10); 
                break;
            }
            case (_fixedTech === 99): {
                let _fixChance = 0.1; 
                break;
            }
            default: {
                let _fixChance = 0.2; 
                break;
            }
        }

        log(this.name, _fname+": fixChance: "+_fixChance);
        if(Math.random() < _fixChance)
            this.$fixItem(_eqInfo); 
        else { 
            log(this.name, _fname+": "+_fixedName + " repair attempt has failed.");
            player.consoleMessage(_fixedName + " repair attempt has failed.", 5); 
        }
    }
}

//------------------------------------------------------------------------------------------//
this.$fixItem = function(eqInfo) {
    var _fname = "fixItem";
    var _ship = player.ship;
    var _eqKey = eqInfo.equipmentKey;

    _ship.setEquipmentStatus(_eqKey,"EQUIPMENT_OK"); // and actually fix the thing!
    log(this.name, _fname+": "+eqInfo.name + " fixed by the repair bots.");
    player.consoleMessage(eqInfo.name + " fixed by the repair bots.", 5)
    switch (_eqKey) { // specific OXP equipment which need rebooting after fixing, or have other issues.
        case "EQ_FRAME_FUEL_COLLECTOR": {
            if (worldScripts["Fuel Collector"]) 
                worldScripts["Fuel Collector"].shipLaunchedFromStation(); // restart the timers in it's world script
            break;
        }
        case "EQ_FRAME_BOUNTY_SCANNER": {
            if (worldScripts["Bounty Scanner"])  
                worldScripts["Bounty Scanner"].shipLaunchedFromStation(); // restart the timers in it's world script
            break;
        }
        case "EQ_EEU": {
            if (worldScripts["Emergency Energy Unit"])
                worldScripts["Emergency Energy Unit"].shipLaunchedFromStation(); // restart the timers in it's world script
            break;
        }
        case "EQ_ROCKHERMIT_SCANNER": { 
            if (worldScripts["rockHermit_Locator"]) {
                if (worldScripts["rockHermit_Locator"].version === "1.2.3")
                    worldScripts["rockHermit_Locator"].shipLaunchedFromStation() // use the inbuild scripting of the OXP to restart it.
                else {
                    let _beacons = system.shipsWithPrimaryRole("rockbeacof"); // list the inactive beacons (no lights or "R" on compass) 
                    if (_beacons.length > 0) { 
                        for (let i=0; i < _beacons.length;i++) { 
                            if (oolite.compareVersion('1.73') <= 0) 
                                _beacons[i].position = _beacons[i].position.multiply(100); //throw the rockbeacof into space... 
                            else 
                                _beacons[i].setPosition(_beacons[i].position.multiply(100)); //throw the rockbeacof into space... 
                            _beacons[i].explode(); //...and blow it up. 
                        } 
                    } 
                    worldScripts["rockHermit_Locator"].buoy = "rockbeacon" 
                    worldScripts["rockHermit_Locator"].addBuoys() // no inactive beacons, so use original code to add active beacons. 
                }
            }
            break; 
        } 
    }
}
