"use strict";
this.name        = "Repair System";
this.author      = "Thargoid";
this.copyright   = "Creative Commons Attribution - Non-Commercial - Share Alike 3.0 license with clauses - see readme.txt.";
this.description = "Damage Control system based on repair bots";
this.version     = "2.11";

this.$repairControllers = [ "EQ_REPAIRBOTS_CONTROLLER",         // 10 charges version
                            "EQ_REPAIRBOTS_CONTROLLER_LIGHT"    // 5 charges version
                          ];
this.$repairRecharges = [ "EQ_REPAIRBOTS_RECHARGE_10",
                          "EQ_REPAIRBOTS_RECHARGE_5"
                        ];
this.$repairControllerCapacity = {
                                    EQ_REPAIRBOTS_CONTROLLER: 10,
                                    EQ_REPAIRBOTS_CONTROLLER_LIGHT: 5
                                 };
this.$repairRechargesCapacity = {
                                    EQ_REPAIRBOTS_RECHARGE_10: 10,
                                    EQ_REPAIRBOTS_RECHARGE_5: 5
                                };
this.$energyCostPerSecond = 1;  // cost of (energy unit)/s while Repair System is active repairing things
                                // self-repair (repairing the REpair System itself) costs double
this.$energyLowThreshold = 64;  // Self Repair System shutdown if energy bellow this
this.$energyResumeThreshold = 128; // Self-Repair System resumes if energy above this
this.$selfRepairsSystemStatus; // ['Stand-by','Active','Self-repair', 'Shutdown']
this.$controllerVersion;    // The version of Self-Repair System Controller installed in the player ship
this.$delayArray;           // list of damaged equipment to be repaired
this.$playerDamagedList = [];
this.$accumEnergyCost = 0;
this.$energyCostCounter = 0;
this.$debug = true;

/****************************************************************************************************/
// Event Handlers 
//--------------------------------------------------------------------------------------------------//
this.startUp = function() { 
    var _ship = player.ship;
    var repairControllers = this.$repairControllers;
    var i;

    this.$controllerVersion = null;
    this.$delayArray = [];
    i = repairControllers.length;
    while (i--) {
        if (_ship.equipmentStatus(repairControllers[i]) === "EQUIPMENT_OK" ||
            _ship.equipmentStatus(repairControllers[i]) === "EQUIPMENT_DAMAGED") {
            this.$controllerVersion = repairControllers[i];
            if (!missionVariables.repairCounter) 
                missionVariables.repairCounter = this.$repairControllerCapacity[repairControllers[i]];
            break;
        }
    }
    if (this.$controllerVersion)
        log(this.name, _ship.displayName+" has Repair System Controller "+this.$controllerVersion+" with "+missionVariables.repairCounter+" charges remaining"); 
}

//--------------------------------------------------------------------------------------------------//
this.playerBoughtEquipment = function(equipment) {
    var _player = player;
    var repairRecharges = this.$repairRecharges;
    var i;

    // deal with the controllers
    this.equipmentAdded(equipment);       
    
    i = repairRecharges.indexOf(equipment);
    if (i >= 0) {
        // player bought a Sefl-Repair System recharge
        var recharge = this.$repairRechargesCapacity[repairRecharges[i]];
        missionVariables.repairCounter += recharge; 
        log(this.name, "Recharging Self-Repair System with "+recharge+" charges");
        _player.ship.removeEquipment(equipment);
        this.$limitCharges();
    } else if (equipment === "EQ_REPAIRBOTS_REMOVER") {
            var refund = this.$calculateRepairSystemRefund();
            var msg = "Refunding "+formatCredits(refund, false, true)+" for "+EquipmentInfo.infoForKey(this.$controllerVersion).name;
            log(this.name, msg);
            _player.consoleMessage(msg, 5);
            _player.credits += refund;
            this.$removeRepairController();
            _player.ship.removeEquipment(equipment);
    }
}

//--------------------------------------------------------------------------------------------------//
this.equipmentAdded = function(equipment) {
    // deals only wjith the controllers, which can be added by script
    var repairControllers = this.$repairControllers;
    var i = repairControllers.indexOf(equipment);

    if (i >= 0) {
        // player bought a Self-Repair System
        this.$controllerVersion = repairControllers[i];
        missionVariables.repairCounter = this.$repairControllerCapacity[repairControllers[i]];
        log(this.name, "Ship awarded Self-Repair System with "+missionVariables.repairCounter+" charges");
    }
}

//--------------------------------------------------------------------------------------------------//
this.playerBoughtNewShip = function(ship) {
    var _ship = player.ship;
    var repairControllers = this.$repairControllers;
    var i;

    this.$controllerVersion = null;    
    i = repairControllers.length;
    while (i--) {
        if (_ship.equipmentStatus(repairControllers[i]) === "EQUIPMENT_OK" ||
            _ship.equipmentStatus(repairControllers[i]) === "EQUIPMENT_DAMAGED") {
            missionVariables.repairCounter = this.$repairControllerCapacity[repairControllers[i]];
            this.$controllerVersion = repairControllers[i];
            break;
        }
    }

    if (this.$controllerVersion == null) 
        this.$removeRepairController();
}
 
//--------------------------------------------------------------------------------------------------//
this.guiScreenWillChange = function(to, from) {
    if (this.$controllerVersion == null) return;
    if (to === "GUI_SCREEN_MANIFEST") {
        switch (this.$getRepairSystemEqStatus()) {
            case "EQUIPMENT_OK": {
                if (missionVariables.repairCounter > 0)
                    this.repairDisplay = "Remaining repair system charges - " + missionVariables.repairCounter;
                else
                    this.repairDisplay = "Repair system offline - recharge required";
                mission.setInstructions(this.repairDisplay); 
                break;
            }
        
            case "EQUIPMENT_DAMAGED": {
                mission.setInstructions("Repair system offline - damaged");
                break;
            }            
            
            case "EQUIPMENT_UNAVAILABLE":
            case "EQUIPMENT_UNKNOWN": {
                this.$removeRepairController();
                break;
            }
        }
    }    
}    

//--------------------------------------------------------------------------------------------------//
this.equipmentDamaged = function(equipment) {
    if (this.$controllerVersion == null) return;
    if (player.ship.docked) return;

    log(this.name, "Equipment "+equipment+" damaged, putting in list for eventual repair");
    if (this.$delayArray.indexOf(equipment) === -1)
        this.$delayArray.push(equipment);
    if (!this.$equipmentDelay)  
        this.$equipmentDelay = new Timer(this, this.$checkEquipment, 0.5, 5); 
    else 
        this.$equipmentDelay.start();
}

//--------------------------------------------------------------------------------------------------//
this.equipmentDestroyed  = function(equipment) {
    if (equipment == this.$controllerVersion) {
        log(this.name, "Self-Repair System Controller destroyed"); 
        this.$removeRepairController();
    } else {
        var eqInfo = EquipmentInfo.infoForKey(equipment);
        log(this.name, "Equipment "+eqInfo.name+" ("+equipment+") destroyed");
    }
}    

//--------------------------------------------------------------------------------------------------//
this.shipWillDockWithStation = this.shipDied = function() {
    this.$stopTimers();
    this.$selfRepairsSystemStatus = 'Shutdown';
}

//--------------------------------------------------------------------------------------------------//
this.shipWillLaunchFromStation = function() {
    this.$startSelfRepairSystem("Launch");
}



/****************************************************************************************************/
// Internal Functions 

//--------------------------------------------------------------------------------------------------//
this.$checkEquipment = function() {
    var _fname = "checkEquipment";

    if(!this.$delayArray || this.$delayArray.length === 0) return;

    // get one equipment from the damaged list
    var equipment = this.$delayArray.pop();
    if(player.ship.equipmentStatus(equipment) !== "EQUIPMENT_DAMAGED") { 
        log(this.name, _fname+": equipment "+EquipmentInfo.infoForKey(equipment).name+" ("+equipment+") has already been repaired");
        return; 
    }
    
    if (equipment == this.$controllerVersion) {
        // Self-Repair Controller damaged
        if (this.$damageControlTimer && this.$damageControlTimer.isRunning) {
            this.$damageControlTimer.stop();
            delete this.$damageControlTimer;
        }
        var repairTime = 60 + Math.ceil(Math.random() * 120);
        log(this.name, _fname+": Repair system entering self-repair mode, repair time "+repairTime);
        player.consoleMessage("Repair system entering self-repair mode", 6);
        log(this.name, _fname+": starting timer for selfRepair");
        this.$selfRepairsSystemStatus = 'Self-repair';
        this.$selfRepairTimer = new Timer(this, this.$selfRepair, repairTime);
        this.$startEnergyConsumption();
    } else {
        // other equipment than the Self-repair System Controller was damaged
        if ((missionVariables.repairCounter && missionVariables.repairCounter > 0) && 
               (!this.$damageControlTimer || !this.$damageControlTimer.isRunning)) {
            // Self-Repair System in standby, activating
            var repairTime = 120 + Math.ceil(Math.random() * 180);
            log(this.name, _fname+": Repair system activated, repair time "+repairTime);
            player.consoleMessage("Repair system activated", 6);
            log(this.name, _fname+": starting timer for repairSystems");
            this.$selfRepairsSystemStatus = 'Active';
            this.$damageControlTimer = new Timer(this, this.$repairSystems, repairTime);
            this.$startEnergyConsumption();
        }
    }
}

//--------------------------------------------------------------------------------------------------//
this.$selfRepair = function() {
    var _fname = "selfRepair";
    var _player = player;

    if (this.$controllerVersion == null ) return;
    
    if(Math.random() < 0.75) {
        // 75% chance of self-repair
        log(this.name, _fname+": Repair System self-repair complete")
        _player.ship.setEquipmentStatus(this.$controllerVersion,"EQUIPMENT_OK");
        this.$checkSystems();
        if ((missionVariables.repairCounter && missionVariables.repairCounter > 0) && 
            this.$playerDamagedList.length > 0) {
            // there are damaged equipments, activate system
            _player.consoleMessage("Repair system online and activated", 6);
            var repairTime = 120 + Math.ceil(Math.random() * 180);
            log(this.name, _fname+": Repair System online and activated, repair time "+repairTime);
            log(this.name, _fname+": starting timer for repairSystems");
            this.$selfRepairsSystemStatus = 'Active';
            this.$damageControlTimer = new Timer(this, this.$repairSystems, repairTime);
            this.$startEnergyConsumption();
        } else {
            this.$stopEnergyConsumption();
            if (missionVariables.repairCounter && missionVariables.repairCounter > 0) { 
                // no damaged equipment
                log(this.name, _fname+": Repair system online and in standby mode");
                _player.consoleMessage("Repair system online and in standby mode", 6); 
                this.$selfRepairsSystemStatus = 'Standby';
            } else {
                mission.setInstructions("Repair system offline - recharge required");
                this.$selfRepairsSystemStatus = 'Shutdown';
            }
        }
     } else {
        if (_player.ship.equipmentStatus(this.$controllerVersion) === "EQUIPMENT_DAMAGED") {
            // Self-repair System Controller still damaged (just in case it was repaired in the meantime)
            var repairTime = 60 + Math.ceil(Math.random() * 120);
            log(this.name, _fname+": Repair system offline - self-repair continuing, repair time "+repairTime);
            player.consoleMessage("Repair system offline - self-repair continuing", 6);
            log(this.name, _fname+": starting timer for selfRepair");
            this.$selfRepairsSystemStatus = 'Self-repair';
            this.$selfRepairTimer = new Timer(this, this.$selfRepair, repairTime);
            this.$startEnergyConsumption();
        }
    }
}

//--------------------------------------------------------------------------------------------------//
this.$checkSystems = function() {
    var _fname = "checkSystems";
    var _ship = player.ship;
    var equipment = _ship.equipment;
    var eqInfo, eq, i;

    this.$playerDamagedList.length = 0;
    i = equipment.length;
    while (i--) {
        eqInfo = equipment[i];
        eq = eqInfo.equipmentKey;    
        if (_ship.equipmentStatus(eq) === "EQUIPMENT_DAMAGED") {
            if (eqInfo.scriptInfo.thargoidRepairBotChance === undefined || 
                isNaN(eqInfo.scriptInfo.thargoidRepairBotChance) ||
                (!isNaN(eqInfo.scriptInfo.thargoidRepairBotChance) && eqInfo.scriptInfo.thargoidRepairBotChance > 0))
                this.$playerDamagedList.push(eq) // if it's broke and fixable, add it to the list.
        }
    }
    log(this.name, _fname+": damaged equipment list: '"+this.$playerDamagedList+"'");
}

//--------------------------------------------------------------------------------------------------//
this.$energyConsumption = function() {
    var _ship = player.ship;
    var _eqStatus;
    var _energyCost;

    if (!this.$controllerVersion) return;
    _eqStatus = _ship.equipmentStatus(this.$controllerVersion);
    if (_eqStatus === "EQUIPMENT_OK")
        _energyCost = this.$energyCostPerSecond / 2;
    else if (_eqStatus === "EQUIPMENT_DAMAGED")
        _energyCost -= this.$energyCostPerSecond;
    _ship.energy -= _energyCost;

    this.$energyCostCounter++;
    this.$accumEnergyCost += _energyCost;
    if (this.$energyCostCounter % 60 === 0) { // 30s
        if (this.$debug) log(this.name, "Energy cost in the last 30s: "+this.$accumEnergyCost.toFixed(1));
        this.$energyCostCounter  = 0;
        this.$accumEnergyCost = 0;
    }

    if (_ship.energy <= this.$energyLowThreshold) {
        log(this.name, "Low energy threshold reached, shutting down Sel-Repair System");
        player.consoleMessage("Low energy threshold reached, shutting down Sel-Repair System", 5);
        this.$selfRepairsSystemStatus = 'Shutdown';
        this.$stopTimers();

        if (!this.$energyResumeTimer)
            this.$energyResumeTimer = new Timer(this, this.$verifyEnergyResume, 0.5, 0.5);
        else if (!this.$energyResumeTimer.isRunning)
            this.$energyResumeTimer.start();
    }
}

//--------------------------------------------------------------------------------------------------//
this.$verifyEnergyResume = function() {
    var _ship = player.ship;

    if (this.$debug) log(this.name, _ship.displayName+" energy:"+_ship.energy.toFixed(0)+", actvation threshold:"+this.$energyResumeThreshold);
    if (player.ship.energy >= this.$energyResumeThreshold) {
        this.$stopTimers();
        this.$startSelfRepairSystem("Resume");
    }
}

//--------------------------------------------------------------------------------------------------//
this.$startSelfRepairSystem = function(context) {
    var _player = player;
    var eqStatus = this.$getRepairSystemEqStatus();
    if (this.$controllerVersion) log(this.name, context+": Repair System equipment status: "+eqStatus);
    if (eqStatus === "EQUIPMENT_OK") {
        this.$checkSystems();
        if (missionVariables.repairCounter && missionVariables.repairCounter > 0 && this.$playerDamagedList.length > 0) {
            // there is equipment damaged and the Self-Repair System has charges
            var repairTime = 120 + Math.ceil(Math.random() * 180);
            log(this.name, context+": Repair system activated");
            _player.consoleMessage("Repair system activated", 6);
            log(this.name, "starting timer for repairSystems");
            this.$selfRepairsSystemStatus = 'Active';
            this.$damageControlTimer = new Timer(this, this.$repairSystems, repairTime);    
            this.$startEnergyConsumption();
        } else if (missionVariables.repairCounter && missionVariables.repairCounter > 0) {
            // Self-repair System has charges
            log(this.name, context+": Repair system Online, no equipment damaged, entering Standby mode");
            _player.consoleMessage("Repair system Online, no equipment damaged, entering Standby mode", 6);
            this.$selfRepairsSystemStatus = 'Standby';
        } else {
            // Self-Repair System charges exausted
            log(this.name, context+": Repair charges exausted ("+missionVariables.repairCounter+"), shutting down");
            _player.consoleMessage("Repair system exhausted - recharge required", 6); 
            this.$selfRepairsSystemStatus = 'Shutdown';
            this.$stopTimers();
        }
    } else {
        var repairTime = 60 + Math.ceil(Math.random() * 120);
        log(this.name, context+": Repair system entering self-repair mode, repair time "+repairTime);
        player.consoleMessage("Repair system entering self-repair mode", 6);
        log(this.name, "starting timer for selfRepair");
        this.$selfRepairsSystemStatus = 'Self-repair';
        this.$selfRepairTimer = new Timer(this, this.$selfRepair, repairTime);    
        this.$startEnergyConsumption();
    }
}

//--------------------------------------------------------------------------------------------------//
this.$repairSystems = function() {
    var _fname = "repairSystems";
    var _player = player;
    var _fixedItem, _fixedName, _fixedTech, _fixChance;
    this.$checkSystems();
    if (this.$playerDamagedList.length === 0) {
        log(this.name, _fname+": Available ship repair completed ("+missionVariables.repairCounter+" charges remain)");
        log(this.name, _fname+": Repair system entering standby mode");
        _player.consoleMessage("Available ship repair completed - Repair system entering standby mode", 5);
        this.$selfRepairsSystemStatus = 'Standby';
        this.$stopEnergyConsumption();
        return;
    } else {
        var damagedEquipment = Math.floor(Math.random() * this.$playerDamagedList.length); // pick a random element from the list...
        _fixedItem = this.$playerDamagedList[damagedEquipment]; // ...define the item...
        _fixedName = EquipmentInfo.infoForKey(_fixedItem).name; // define it's screen name
        _fixedTech = EquipmentInfo.infoForKey(_fixedItem).effectiveTechLevel // tech level of the item

        if (_player.ship.equipmentStatus(_fixedItem) !== "EQUIPMENT_DAMAGED") {
            log(this.name, _fname+": Equipment random-choosen for repair ("+_fixedItem+") is already repaired, picking another");
            this.$repairSystems();
            return;
        } 
         
        var _eqInfo = EquipmentInfo.infoForKey(_fixedItem);
        switch(true) {
            case (_eqInfo.scriptInfo.thargoidRepairBotChance !== undefined && !isNaN(_eqInfo.scriptInfo.thargoidRepairBotChance)): {
                _fixChance = parseFloat(_eqInfo.scriptInfo.thargoidRepairBotChance);
                break;
            }
            case (_fixedTech < 9): {
                _fixChance = 1; 
                break;
            }
            case ((_fixedTech > 8) && (_fixedTech < 17)): {
                _fixChance = 1 - ((_fixedTech - 8)/10); 
                break;
            }
            case (_fixedTech == 99): {
                _fixChance = 0.1; 
                break;
            }
            default: {
                _fixChance = 0.2; 
                break;
            }
        }    
        var _rnd = Math.random();
        log(this.name, _fname+": probability of fixing "+_fixedName+" is "+_fixChance.toFixed(3)+", random: "+_rnd.toFixed(3));    
        if(_rnd < _fixChance)
            this.$fixItem(_eqInfo);
        else {
            if (missionVariables.repairCounter && missionVariables.repairCounter > 0) {
                var repairTime = 120 + Math.ceil(Math.random() * 180);
                _player.consoleMessage(_fixedName + ": repair attempt failed - work continuing.", 5);
                log(this.name, _fname+": repair attempt of "+_fixedItem+" failed, trying again in "+repairTime);
                log(this.name, "starting timer for repairSystems");
                this.$selfRepairsSystemStatus = 'Active';
                this.$damageControlTimer = new Timer(this, this.$repairSystems, repairTime);
                this.$startEnergyConsumption();
            } else {
                log(this.name, _fname+": Repair charges exausted ("+missionVariables.repairCounter+") while trying to repair "+_fixedItem+", damaged equipment:"+this.$playerDamagedList);
                _player.consoleMessage("Repair system exhausted - recharge required", 6); 
                this.$selfRepairsSystemStatus = 'Shutdown';
                this.$stopEnergyConsumption();
            }
        }
    }
}

//--------------------------------------------------------------------------------------------------//
this.$fixItem = function(eqInfo) {
    var _fname = "fixItem";
    var _player = player;
    var _eq = eqInfo.equipmentKey;

    if (missionVariables.repairCounter > 0) {
        _player.ship.setEquipmentStatus(_eq,"EQUIPMENT_OK"); // and actually fix the thing!
        missionVariables.repairCounter--;
        log(this.name, _fname+": "+eqInfo.name + " online and operational, deducted one charge ("+missionVariables.repairCounter+" remain)");
        _player.consoleMessage(eqInfo.name + " online and operational", 5);
        switch (_eq) {
            // specific OXP equipment which need rebooting after fixing, or have other issues.
            case "EQ_FRAME_FUEL_COLLECTOR": {
                if (worldScripts["Fuel Collector"])
                    worldScripts["Fuel Collector"].shipLaunchedFromStation(); // restart the timers in it's world script
                break;
            }
            case "EQ_FRAME_BOUNTY_SCANNER": {
                if (worldScripts["Bounty Scanner"])
                    worldScripts["Bounty Scanner"].shipLaunchedFromStation(); // restart the timers in it's world script
                break;
            }
            case "EQ_EEU": {
                if (worldScripts["Emergency Energy Unit"])
                    worldScripts["Emergency Energy Unit"].shipLaunchedFromStation(); // restart the timers in it's world script
                break;
            }
            case "EQ_ROCKHERMIT_SCANNER": { 
                if (worldScripts["rockHermit_Locator"]) {
                    if(worldScripts["rockHermit_Locator"].version === "1.2.3")
                        worldScripts["rockHermit_Locator"].shipLaunchedFromStation() // use the inbuild scripting of the OXP to restart it.
                    else {
                        this.$beacons = system.shipsWithPrimaryRole("rockbeacof"); // list the inactive beacons (no lights or "R" on compass) 
                        if (this.$beacons.length > 0) { 
                            for (let i=0; i<this.$beacons.length;i++) { 
                                if (oolite.compareVersion('1.73') <= 0) 
                                    this.$beacons[i].position = this.$beacons[i].position.multiply(100); //throw the rockbeacof into space... 
                                else 
                                    this.$beacons[i].setPosition(this.$beacons[i].position.multiply(100)); //throw the rockbeacof into space... 
                                this.$beacons[i].explode(); //...and blow it up. 
                            } 
                        } 
                        worldScripts["rockHermit_Locator"].buoy = "rockbeacon" 
                        worldScripts["rockHermit_Locator"].addBuoys() // no inactive beacons, so use original code to add active beacons. 
                    }
                } 
                break; 
            }
        }
        if (this.$playerDamagedList.length > 1) {
            if (missionVariables.repairCounter && missionVariables.repairCounter > 0) {
                var repairTime = 120 + Math.ceil(Math.random() * 180);
                log(this.name, _fname+": Further damage detected - repairs continuing, repair time "+repairTime);
                _player.consoleMessage("Further damage detected - repairs continuing", 6);
                log(this.name, "starting timer for repairSystems");
                this.$selfRepairsSystemStatus = 'Active';
                this.$damageControlTimer = new Timer(this, this.$repairSystems, repairTime);
                this.$startEnergyConsumption();    
            } else  { 
                log(this.name, _fname+": Repair system exhausted - recharge required"); 
                _player.consoleMessage("Repair system exhausted - recharge required", 6); 
                this.$selfRepairsSystemStatus = 'Shutdown';
                this.$stopEnergyConsumption();
            }
        } else {
            log(this.name, _fname+": No further damage, Repair system entering standby mode");
            _player.consoleMessage("No further damage, Repair system entering standby mode", 6); 
            this.$selfRepairsSystemStatus = 'Standby';
            this.$stopEnergyConsumption();
        }
    }
}

//--------------------------------------------------------------------------------------------------//
this.$startEnergyConsumption = function() {
    if (!this.$energyConsumptionTimer)
        this.$energyConsumptionTimer = new Timer(this, this.$energyConsumption, 0.5, 0.5);
    else if (!this.$energyConsumptionTimer.isRunning)
        this.$energyConsumptionTimer.start();
}

//--------------------------------------------------------------------------------------------------//
this.$stopEnergyConsumption = function() {
    if (this.$energyConsumptionTimer && this.$energyConsumptionTimer.isRunning) {
        this.$energyConsumptionTimer.stop();
        delete this.$energyConsumptionTimer;
        if (this.$debug) log(this.name, "Energy cost in the last "+(this.$energyCostCounter/2).toFixed(1)+"s: "+this.$accumEnergyCost.toFixed(1));
        this.$energyCostCounter  = 0;
        this.$accumEnergyCost = 0;
    }
}

//--------------------------------------------------------------------------------------------------//
this.$stopTimers = function() { 
    if(this.$damageControlTimer && this.$damageControlTimer.isRunning) {
        this.$damageControlTimer.stop();
        delete this.$damageControlTimer;
    }
    if (this.$selfRepairTimer && this.$selfRepairTimer.isRunning) {
        if (this.$controllerVersion) player.ship.setEquipmentStatus(this.$controllerVersion,"EQUIPMENT_OK");
        this.$selfRepairTimer.stop();
        delete this.$selfRepairTimer;
    }    
    if (this.$equipmentDelay && this.$equipmentDelay.isRunning) {
        this.$equipmentDelay.stop();
        delete this.$equipmentDelay;
    }
    if (this.$energyConsumptionTimer && this.$energyConsumptionTimer.isRunning) {
        this.$energyConsumptionTimer.stop();
        delete this.$energyConsumptionTimer;
    }
    if (this.$energyResumeTimer && this.$energyResumeTimer.isRunning) {
        this.$energyResumeTimer.stop();
        delete this.$energyResumeTimer;
    }
}

//--------------------------------------------------------------------------------------------------//
this.$getRepairSystemEqStatus = function() {
    if (this.$controllerVersion == null) return null;
    return (player.ship.equipmentStatus(this.$controllerVersion))
}

//--------------------------------------------------------------------------------------------------//
this.$limitCharges = function() {
    var limit = (this.$controllerVersion ? this.$repairControllerCapacity[this.$controllerVersion] : null);
    if (limit == null) {
        if (missionVariables.repairCounter) delete missionVariables.repairCounter;
    } else if (missionVariables.repairCounter > limit)
        missionVariables.repairCounter = limit;
}

//--------------------------------------------------------------------------------------------------//
this.$removeRepairController = function() {
    if (this.$controllerVersion) player.ship.removeEquipment(this.$controllerVersion);
    this.$controllerVersion = null;
    if (missionVariables.repairCounter) delete missionVariables.repairCounter;
    this.$stopTimers(); 
    mission.setInstructions(null);
}

//--------------------------------------------------------------------------------------------------//
this.$calculateRepairSystemRefund = function() {
    var _ship = player.ship;
    var eq = this.$controllerVersion;
    var eqInfo = EquipmentInfo.infoForKey(eq);
    var refund;

    if (_ship.equipmentStatus(eq) === "EQUIPMENT_OK") {
        refund = eqInfo.price * 0.80 / 10;
        log(this.name, "Refund for "+eqInfo.name+" ("+eq+") calculated at "+refund);
        return refund;
    }
    return 0;
}

